@@ -1,6 +1,7 @@
+#define IN_LIBEXSLT
 #include "libexslt/libexslt.h"
 
-#if defined(WIN32) && !defined (__CYGWIN__)
+#if defined(WIN32) && !defined (__CYGWIN__) && (!__MINGW32__)
 #include <win32config.h>
 #else
 #include "config.h"
@@ -33,14 +34,12 @@ exsltSetsDifferenceFunction (xmlXPathParserContextPtr ctxt, int nargs) {
     }
 
     arg2 = xmlXPathPopNodeSet(ctxt);
-    if (xmlXPathCheckError(ctxt)) {
-	xmlXPathSetTypeError(ctxt);
+    if (xmlXPathCheckError(ctxt))
 	return;
-    }
 
     arg1 = xmlXPathPopNodeSet(ctxt);
     if (xmlXPathCheckError(ctxt)) {
-	xmlXPathSetTypeError(ctxt);
+        xmlXPathFreeNodeSet(arg2);
 	return;
     }
 
@@ -70,14 +69,12 @@ exsltSetsIntersectionFunction (xmlXPathParserContextPtr ctxt, int nargs) {
     }
 
     arg2 = xmlXPathPopNodeSet(ctxt);
-    if (xmlXPathCheckError(ctxt)) {
-	xmlXPathSetTypeError(ctxt);
+    if (xmlXPathCheckError(ctxt))
 	return;
-    }
 
     arg1 = xmlXPathPopNodeSet(ctxt);
     if (xmlXPathCheckError(ctxt)) {
-	xmlXPathSetTypeError(ctxt);
+        xmlXPathFreeNodeSet(arg2);
 	return;
     }
 
@@ -98,13 +95,22 @@ exsltSetsIntersectionFunction (xmlXPathParserContextPtr ctxt, int nargs) {
  */
 static void
 exsltSetsDistinctFunction (xmlXPathParserContextPtr ctxt, int nargs) {
+    xmlXPathObjectPtr obj;
     xmlNodeSetPtr ns, ret;
+    int boolval = 0;
+    void *user = NULL;
 
     if (nargs != 1) {
 	xmlXPathSetArityError(ctxt);
 	return;
     }
 
+    if (ctxt->value != NULL) {
+        boolval = ctxt->value->boolval;
+	user = ctxt->value->user;
+	ctxt->value->boolval = 0;
+	ctxt->value->user = NULL;
+    }
     ns = xmlXPathPopNodeSet(ctxt);
     if (xmlXPathCheckError(ctxt))
 	return;
@@ -112,9 +118,13 @@ exsltSetsDistinctFunction (xmlXPathParserContextPtr ctxt, int nargs) {
     /* !!! must be sorted !!! */
     ret = xmlXPathDistinctSorted(ns);
 
-    xmlXPathFreeNodeSet(ns);
+	if (ret != ns)
+		xmlXPathFreeNodeSet(ns);
 
-    xmlXPathReturnNodeSet(ctxt, ret);
+    obj = xmlXPathWrapNodeSet(ret);
+    obj->user = user;
+    obj->boolval = boolval;
+    valuePush((ctxt), obj);
 }
 
 /**
@@ -136,14 +146,12 @@ exsltSetsHasSameNodesFunction (xmlXPathParserContextPtr ctxt,
     }
 
     arg2 = xmlXPathPopNodeSet(ctxt);
-    if (xmlXPathCheckError(ctxt)) {
-	xmlXPathSetTypeError(ctxt);
+    if (xmlXPathCheckError(ctxt))
 	return;
-    }
 
     arg1 = xmlXPathPopNodeSet(ctxt);
     if (xmlXPathCheckError(ctxt)) {
-	xmlXPathSetTypeError(ctxt);
+        xmlXPathFreeNodeSet(arg2);
 	return;
     }
 
@@ -172,14 +180,12 @@ exsltSetsLeadingFunction (xmlXPathParserContextPtr ctxt, int nargs) {
     }
 
     arg2 = xmlXPathPopNodeSet(ctxt);
-    if (xmlXPathCheckError(ctxt)) {
-	xmlXPathSetTypeError(ctxt);
+    if (xmlXPathCheckError(ctxt))
 	return;
-    }
 
     arg1 = xmlXPathPopNodeSet(ctxt);
     if (xmlXPathCheckError(ctxt)) {
-	xmlXPathSetTypeError(ctxt);
+	xmlXPathFreeNodeSet(arg2);
 	return;
     }
 
@@ -219,14 +225,12 @@ exsltSetsTrailingFunction (xmlXPathParserContextPtr ctxt, int nargs) {
     }
 
     arg2 = xmlXPathPopNodeSet(ctxt);
-    if (xmlXPathCheckError(ctxt)) {
-	xmlXPathSetTypeError(ctxt);
+    if (xmlXPathCheckError(ctxt))
 	return;
-    }
 
     arg1 = xmlXPathPopNodeSet(ctxt);
     if (xmlXPathCheckError(ctxt)) {
-	xmlXPathSetTypeError(ctxt);
+	xmlXPathFreeNodeSet(arg2);
 	return;
     }
 
@@ -276,3 +280,45 @@ exsltSetsRegister (void) {
 				   EXSLT_SETS_NAMESPACE,
 				   exsltSetsTrailingFunction);
 }
+
+/**
+ * exsltSetsXpathCtxtRegister:
+ *
+ * Registers the EXSLT - Sets module for use outside XSLT
+ */
+int
+exsltSetsXpathCtxtRegister (xmlXPathContextPtr ctxt, const xmlChar *prefix)
+{
+    if (ctxt
+        && prefix
+        && !xmlXPathRegisterNs(ctxt,
+                               prefix,
+                               (const xmlChar *) EXSLT_SETS_NAMESPACE)
+        && !xmlXPathRegisterFuncNS(ctxt,
+                                   (const xmlChar *) "difference",
+                                   (const xmlChar *) EXSLT_SETS_NAMESPACE,
+                                   exsltSetsDifferenceFunction)
+        && !xmlXPathRegisterFuncNS(ctxt,
+                                   (const xmlChar *) "intersection",
+                                   (const xmlChar *) EXSLT_SETS_NAMESPACE,
+                                   exsltSetsIntersectionFunction)
+        && !xmlXPathRegisterFuncNS(ctxt,
+                                   (const xmlChar *) "distinct",
+                                   (const xmlChar *) EXSLT_SETS_NAMESPACE,
+                                   exsltSetsDistinctFunction)
+        && !xmlXPathRegisterFuncNS(ctxt,
+                                   (const xmlChar *) "has-same-node",
+                                   (const xmlChar *) EXSLT_SETS_NAMESPACE,
+                                   exsltSetsHasSameNodesFunction)
+        && !xmlXPathRegisterFuncNS(ctxt,
+                                   (const xmlChar *) "leading",
+                                   (const xmlChar *) EXSLT_SETS_NAMESPACE,
+                                   exsltSetsLeadingFunction)
+        && !xmlXPathRegisterFuncNS(ctxt,
+                                   (const xmlChar *) "trailing",
+                                   (const xmlChar *) EXSLT_SETS_NAMESPACE,
+                                   exsltSetsTrailingFunction)) {
+        return 0;
+    }
+    return -1;
+}
